class Similarity():

    def __init__(self):
        pass

    def compare(self, dict1, dict2):
        """
        :param dict1: a dictionnary containing {class_name:occurence}
        :param dict2: same

        :return: a similarity score between 0 and 1, 1 being the maximum similarity value
        (ideally 1 if and only if dict1==dict2)
        """
        return 0
