from vizuka.similarity import similarity

class AllFrontiers(similarity.Similarity):
    def __init__(self):
        pass
    def compare(dict0, dict1, force_inside01=True):
        """
        Draw all frontiers
        """
        return 0

class NoneFrontiers(similarity.Similarity):
    def __init__(self):
        pass
    def compare(dict0, dict1, force_inside01=True):
        """
        Draw none frontiers
        """
        return 1

